package cc.smtweb.biz.mztb.web.task;

import cc.smtweb.biz.mztb.web.base.DbSourceConfig;
import cc.smtweb.biz.mztb.web.base.PartyInfo;
import cc.smtweb.biz.mztb.web.tool.DbUtil;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.system.bpm.web.sys.base.job.BaseJob;
import cc.smtweb.system.bpm.web.sys.user.party.Party;
import cc.smtweb.system.bpm.web.sys.user.party.PartyCache;
import cc.smtweb.system.bpm.web.sys.user.user.User;
import cc.smtweb.system.bpm.web.sys.user.user.UserCache;
import cc.smtweb.system.bpm.web.sys.user.user.UserParty;
import cc.smtweb.system.bpm.web.sys.user.user.UserPartyCache;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.alibaba.druid.proxy.jdbc.ConnectionProxy;
import lombok.SneakyThrows;

import java.lang.invoke.SwitchPoint;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public class AddNewPartyAndAccountTask extends BaseJob {

    @Override
    protected String work() {
        // 查询前置库推送的机构和账号信息
        DbEngine instance = DbEngine.getInstance();
        DbSourceConfig dbSourceConfig = instance.findDao(DbSourceConfig.class).queryEntityWhere("dsc_use_state = 0");
        Connection connection = null;
        ResultSet resultSet = null;
        // 批量处理的数据
        List<PartyInfo> partysUpdateList = new ArrayList<>();
        List<PartyInfo> partysInsertList = new ArrayList<>();
        List<Party> sysPartyList = new ArrayList<>();
        List<User> userList = new ArrayList<>();
        List<UserParty> userPartyList = new ArrayList<>();

        // 记录重复数据
        List<PartyInfo> partysInsertListExist = new ArrayList<>();
        List<Party> sysPartyListExist = new ArrayList<>();
        List<User> userListExist = new ArrayList<>();
        List<UserParty> userPartyListExist = new ArrayList<>();

        SwMap existPartyMap = new SwMap();
        int insertIndex = 0;
        // 使用社会信用代码 当做key
        Map<String, PartyInfo> partyInfoMap = instance.query(PartyInfo.class).stream().collect(Collectors.toMap(PartyInfo::getCreditCode, Function.identity()));
        try {
            connection = DriverManager.getConnection(dbSourceConfig.getWholeUrl(), dbSourceConfig.getUsername(), dbSourceConfig.getPassword());
            DbUtil dbUtil = new DbUtil(connection);
            resultSet = dbUtil.dbHandSql("select * from pre_party_info where state = 0 order by wr_time", "query"); // 查询未读取的机构信息,时间升序 先推送的先读取。
            if (resultSet == null) return null;

            while (resultSet.next()) {
                PartyInfo partyInfo = new PartyInfo();
                SwMap swMap = new SwMap();
                ResultSetMetaData metaData = resultSet.getMetaData();
                int columnCount = metaData.getColumnCount();
                for (int i = 1; i <= columnCount; i++) {
                    swMap.put(metaData.getColumnName(i), resultSet.getString(i));
                }
                // 将查询到的数据转为 填报系统机构表数据
                partyInfo.getData().putAll(swMap);

                if (partyInfoMap.containsKey(resultSet.getString("pi_credit_code"))) { // 判断该机构是否存在
                    long piIdOld = partyInfoMap.get(resultSet.getString("pi_credit_code")).getId();
                    long sysPartyId = partyInfoMap.get(resultSet.getString("pi_credit_code")).getSysPartyId();
                    partyInfo.setId(piIdOld);
                    partyInfo.setSysPartyId(sysPartyId);
                    partysUpdateList.add(partyInfo);
                } else { // 新增(包括 底层的party 填报系统的 partyInfo 默认的用户)
                    boolean existParty = existPartyMap.containsKey(resultSet.getString("pi_credit_code"));
                    if (existParty) {
                        // 包含就更新机构信息 ,系统机构信息没有可更改的内容故不处理
                        int index = existPartyMap.readInt(resultSet.getString("pi_credit_code"));
                        partysInsertListExist.add(partysInsertList.get(index));
                        userListExist.add(userList.get(index));
                        userPartyListExist.add(userPartyList.get(index));
                        sysPartyListExist.add(sysPartyList.get(index));
                    }
                    existPartyMap.put(resultSet.getString("pi_credit_code"), insertIndex);
                    insertIndex++;

                    Party party = new Party(); // 底层机构表
                    party.init();
                    long ptId = resultSet.getLong("pi_id"); // 使用前置库的机构id当做 填报系统机构id
                    party.setId(ptId);
                    party.setCode(ptId + "");
                    party.setName(partyInfo.getPartyName());
                    partyInfo.setSysPartyId(ptId);
                    sysPartyList.add(party);
                    partysInsertList.add(partyInfo);

                    User user = new User(); // 底层用户表
                    user.init();
                    long userId = instance.nextId();
                    user.setId(userId);
                    user.setCode(partyInfo.getCreditCode());
                    user.setName(partyInfo.getPartyName() + "-填报人员");
                    user.setNicky(partyInfo.getCreditCode());
                    user.setLevel(100);
                    user.setStatu(1);
                    user.setPwd("13217b04835b4cbceb55edb54a72deb4"); //默认密码 abc@123456
                    user.setPwdLevel(0);
                    userList.add(user);

                    // todo 添加部门

                    UserParty userParty = new UserParty(); //用户机构关联表
                    userParty.setId(instance.nextId());
                    userParty.setPartyId(ptId);
                    userParty.setUserId(userId);
                    userParty.setDeptId(869944043285516288L); // 关联部门表
                    userPartyList.add(userParty);

                }
            }
            // 删除重复新增的元素
            partysInsertList.removeAll(partysInsertListExist);
            sysPartyList.removeAll(sysPartyListExist);
            userList.removeAll(userListExist);
            userPartyList.removeAll(userPartyListExist);

            // 读取完成后 修改前置库推送数据的状态，再进行本系统数据入库处理
            dbUtil.update("update pre_party_info set state = 1 , rd_time = " + DateUtil.nowDateTimeNumber() + " where state = 0");

            instance.doTrans(() -> {
                // 处理新增或者修改的数据
                instance.findDao(PartyInfo.class).batchInsertEntity(partysInsertList);
                instance.findDao(PartyInfo.class).batchUpdateEntity(partysUpdateList);
                instance.findDao(Party.class).batchInsertEntity(sysPartyList);
                instance.findDao(User.class).batchInsertEntity(userList);
                instance.findDao(UserParty.class).batchInsertEntity(userPartyList);
                // 将机构、用户信息放入缓存
                PartyCache.getInstance().putList(sysPartyList);
                UserCache.getInstance().putList(userList);
                UserPartyCache.getInstance().putList(userPartyList);
            });
        } catch (SQLException e) {
            e.printStackTrace();
        }
        // 判断是新增还是修改（如果是修改 将机构信息删除 重新新增，注意维护系统机构表数据）
        // 新增的机构 自动创建一个登录账号
        //读取完成后 修改前置库 推送机构表的状态和读取时间
        // 在当前系统 建立机构和账号数据,将账号信息写入缓存
        return null;
    }
}
